#include <stdio.h>
//C++ source file - Open Producer - Copyright (C) 2002 Don Burns
//Distributed under the terms of the GNU LIBRARY GENERAL PUBLIC LICENSE (LGPL)
//as published by the Free Software Foundation.

// Simple example of use of Producer::RenderSurface
// The myGraphics class is a simple sample of how one would implement
// graphics drawing with Producer::RenderSurface

#include <Producer/Camera>
#include <Producer/KeyboardMouse>
#include <Producer/Trackball>

#include "MySceneHandler2"
#include "MyKeyboardMouseCallback"

int main()
{
	Producer::Camera camera;
	Producer::RenderSurface *rs = camera.getRenderSurface();
	rs->setWindowRectangle( 100, 100, 640, 480 );
	rs->setWindowName( "Producer Example using Camera" );

	MySceneHandler sh;
	camera.setSceneHandler( &sh );

	Producer::KeyboardMouse kbm(rs);
	bool done = false;
	MyKeyboardMouseCallback kbmcb(done);
	kbm.setCallback( &kbmcb );
	kbm.startThread();

	Producer::Trackball tb;
	tb.setOrientation( Producer::Trackball::Y_UP );

	while( !done )
	{
		tb.input( kbmcb.mx(), kbmcb.my(), kbmcb.mbutton() );
		sh.setMatrix( tb.getMatrix().ptr() );
		// Place the Camera into the world
		camera.setViewByLookat( 0.0f, 0.0f, 8.0f,   /// Position of the eye
						0.0f, 0.0f, 0.0f,   /// Point the eye is looking at
						0.0f, 1.0f, 0.0f ); /// Up
		camera.frame();
	}
	return 0;
}
